## Plankton-classifier analysis of Phytodive CPICS images 

Here we document the training, evaluation and application of the Plankton-classifier CNN to CPICS images acquired for the Phytodive project during the Ludwig-Prandtl cruises of 2022. 

## Finetuning a CNN on randomly-labeled images

1. The dataset, to be made public upon publication, was imported into a [Label-studio](https://labelstud.io) project for annotation (random sampling). First, a set of [3022 labels](../manual_annotations_data/phytodive_labels_for_CNN_project-1-at-2022-10-26-07-54.csv) were generated by domain experts. The class distribution of randomly labeled images showed was unbalanced, as expected from existing knowledge on the prevalence of various marine organisms and particle types. The 'blurry' and 'marine snow' classes were predominant. 

![classDist_project-1-at-2022-10-26-07-54](../figures/phytodive_labels_for_CNN_project-1-at-2022-10-26-07-54.png)


2. For such unbalanced data, typical in of aquatic _in situ_ images, a semi-supervised machine learning paradigm was [previously shown](https://arxiv.org/abs/2002.05709) to enhance CNN performance. In our preliminary analysis, we used weights of a previously carried-out, unsupervised pre-training with our custom implemented SimCLR feature extractor with 100,000 images from a Helgoland observatory (Schanz et al., in review). 

3. The pretrained CNN was trained further on Phytodive [labels](../manual_annotations_data/phytodive_labels_for_CNN_project-1-at-2022-10-26-07-54.csv), with 50% used for training and 50% for validation. A Jupyter notebook for splitting manual annotations into training and validation is [provided](../notebooks/split_labels_into_training_validation_test.ipynb). 
Working version of CNN (setup: finetune_sgd) evaluated on validation data achieved 0.88 accuracy that was consistent across different random initializations:

![CNN_training1_Phytodive_randominits](../figures/CNN_training1_Phytodive_randominits.png)

4. A confusion matrix, showing counts for all possible combinations of true avs. predicted class labels, was computed for the validation data. This showed that the CNN over-predicts the 'blurry' class, as many 'Noctiluca', 'marine snow' and 'other' labels were wrongly predicted as 'blurry'. [Code](../notebooks/generate_confusion_matrices_to_evaluate_CNN_predictions.ipynb) to generate such a matrix is provided in this repo.

![Confusion_matrix](../figures/Confusion_matrix_preliminary_analysis.png)

## CNN run on unseen Phytodive data

1. The trained CNN was used to infer class labels for the entire Phytodive dataset (~720,000 unseen images). Class distributions of the predictions are shown below:
![CNN_predictions_PhytodiveCPICS_rois_20221107_135029](../figures/CNN_predictions_PhytodiveCPICS_rois_20221107_135029.png)

2. Predictions were imported into Label-studio as a new project Phytodive_CNN_eval. Evaluation of predictions (~3500 images, random sampling) by human users in Label-studio resulted in the generation of new [class labels](../manual_annotations_data/CNN_eval_rdmqueue_project-4-at-2022-11-23-09-54-a160f045.csv) and a further evaluation of CNN performance. The class distribution of new labels (correct CNN predictions and corrected human annotations) is shown below.
![classDist_CNN_eval_rdmqueue_project-4-at-2022-11-23-09-54](../figures/classDist_CNN_eval_rdmqueue_project-4-at-2022-11-23-09-54.png)

3. Human evaluation on unseen data resulted in an accuracy of 84.7% 
![Right_CNN_eval_rdmqueue_project-4-at-2022-11-23-09-54-a160f045](../figures/human_eval_cnn.png)

## Improving CNN performance with a human-in-the-loop approach (active learning)

1. For each image, the CNN outputs probabilities for each class. The CNN's prediction is the class with the highest probability (max_p). We used the max_p values to estimate the confidence of CNN predictions. Shown below is the distribution of max_p values for the entire dataset:
 
![max_p__distribution](../figures/max(p)_distribution.png)

2. As can be seen from the distribution of max_p values, images with low max_p values (below 0.4) were rare (~8000 images). 

3. All predictions with low certainty (max_p < 0.4, ~8000 images) evaluated by human users in Label-studio. CNN accuracy on the low max_p predictions was low (74% of predictions were wrong), demonstrating that the CNN was less accurate when the probability value assigned to the predicted class was low. 
![cnn_eval_rareclasses](../figures/human_eval_cnn_rareclasses.png)
4. Furthermore, the manual inspections of these images resulted in the discovery of many new classes new classes. Appendicularia, Chaetognates, Copepods, Diatom chains and Pluteus larvae were particularly abundant:
![class_distribution_maxp0.4](../figures/rareclasses_classdist.png)

5. Manual annotations of predictions of max_p values <0.4 are stored [here](../manual_annotations_data/Rare_class_labels_combined_lessthan0.4.csv).

6. The manual annotations generated thus far (randomly sampled labels ~ 7000 and low max_p predictions ~8000) were pooled together and the CNN was finetuned on these 15000 labels (70% training data, 30% validation data). As the number of classes increased doubled from 9 classes to 18 classes, the classification accuracy of the CNN dropped from 88% to 73%. 
![cnn_eval_round2](../figures/CNN_eval2TensorBoard_Accuracy_validation.png).
7. Confusion matrix on the validation data shows that the major source of errors is still Blurries (according to human annotations) that were mislabeled as Marine snow by the CNN.
![confucion_matrix_round2](../figures/Confusion_matrix_validation_cnn_round2.png)
8. Furthermore, we compared pretraining on the Phytodive data (previously carried-out, unsupervised training with our custom implemented SimCLR feature extractor with all 700,000 images from the datset) with transfer-learning (weights used in the preliminary analysis.
While, pretraining (cyan) or transfer-learning (magenta) both significantly improved CNN performance, on our validation data, the two methods performed comparably. This highlights the advantage of obviating computationally expensive pretraining in some cases, in favor of using previously learned features from another dataset.
![overall_accuracy_comp](../figures/2023-02-27_09-52-08_TensorBoard_overall_accuracy_validation.png). 
9. However, conditional accuracies of the various classes of interest (Copepods, Noctiluca, Pluteus larvae, Appendicularia, were considerably improved.
![conditional_accuracies](../figures/2023-03-01_13-12-38_TensorBoard_overview_condacc.png)
10. In this way, we achieved high accuracied on classes of interest, allowing us to extracted from the data for further steps of trait annotation (documentation and code will be made available soon).